"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _WebRTCStats_instances, _WebRTCStats_fillStatObject, _WebRTCStats_getCurrentSDP, _WebRTCStats_getCurrentCodecAndSampleRate, _WebRTCStats_getBytesCount, _WebRTCStats_getBitrate;
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebRTCStats = void 0;
const logger_1 = __importDefault(require("./logger"));
const constants_1 = require("./constants");
class WebRTCStats {
    constructor(pc) {
        _WebRTCStats_instances.add(this);
        this.pc = pc;
        this.logger = new logger_1.default();
        this.tracks = {};
    }
    getStats(track, type, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let statsObject = {};
            if (!track) {
                throw new Error("Can't get statistics for track " + track);
            }
            if (!this.tracks[track.id]) {
                this.tracks[track.id] = { track: track, type: type, bytesCount: 0, seconds: Math.round(new Date().getTime() / 1000) };
            }
            let stat = yield this.pc.getStats(track);
            if (stat) {
                stat.forEach((report) => {
                    if (!report.isRemote) {
                        if (report.type == type) {
                            statsObject = __classPrivateFieldGet(this, _WebRTCStats_instances, "m", _WebRTCStats_fillStatObject).call(this, track.id, report);
                        }
                    }
                });
            }
            callback(statsObject);
        });
    }
}
exports.WebRTCStats = WebRTCStats;
_WebRTCStats_instances = new WeakSet(), _WebRTCStats_fillStatObject = function _WebRTCStats_fillStatObject(id, report) {
    let statObject = {};
    let codec = __classPrivateFieldGet(this, _WebRTCStats_instances, "m", _WebRTCStats_getCurrentCodecAndSampleRate).call(this, report.mediaType);
    let bitrate = __classPrivateFieldGet(this, _WebRTCStats_instances, "m", _WebRTCStats_getBitrate).call(this, id, report);
    statObject["codec"] = codec.name;
    statObject["sampleRate"] = codec.sampleRate;
    if (bitrate > -1) {
        statObject["bitrate"] = bitrate;
    }
    Object.keys(report).forEach((key) => {
        statObject[key] = report[key];
    });
    return statObject;
}, _WebRTCStats_getCurrentSDP = function _WebRTCStats_getCurrentSDP() {
    let description = this.pc.currentRemoteDescription != undefined ? this.pc.currentRemoteDescription : this.pc.remoteDescription;
    let sdp = "";
    if (description && description.sdp) {
        sdp = description.sdp;
    }
    return sdp;
}, _WebRTCStats_getCurrentCodecAndSampleRate = function _WebRTCStats_getCurrentCodecAndSampleRate(mediaType) {
    let sdp = __classPrivateFieldGet(this, _WebRTCStats_instances, "m", _WebRTCStats_getCurrentSDP).call(this);
    let rows = sdp.split("\n");
    let codecPt;
    let result = { name: "undefined", sampleRate: "undefined" };
    for (let i = 0; i < rows.length; i++) {
        if (codecPt && rows[i].indexOf("a=rtpmap:" + codecPt) != -1) {
            result.name = rows[i].split(" ")[1].split("/")[0];
            result.sampleRate = rows[i].split(" ")[1].split("/")[1];
            return result;
        }
        if (rows[i].indexOf("m=" + mediaType) != -1) {
            codecPt = rows[i].split(" ")[3].trim();
        }
    }
    return result;
}, _WebRTCStats_getBytesCount = function _WebRTCStats_getBytesCount(type, report) {
    let bytesCount = 0;
    if (type == constants_1.StatsType.INBOUND) {
        bytesCount = report["bytesReceived"];
    }
    else if (type == constants_1.StatsType.OUTBOUND) {
        bytesCount = report["bytesSent"];
    }
    return bytesCount;
}, _WebRTCStats_getBitrate = function _WebRTCStats_getBitrate(id, report) {
    let bitrate = -1;
    if (this.tracks[id]) {
        let currentBytesCount = __classPrivateFieldGet(this, _WebRTCStats_instances, "m", _WebRTCStats_getBytesCount).call(this, this.tracks[id].type, report);
        let currentSeconds = Math.round(new Date().getTime() / 1000);
        let timePassed = currentSeconds - this.tracks[id].seconds;
        if (timePassed > 0) {
            bitrate = ((currentBytesCount - this.tracks[id].bytesCount) * 8) / timePassed;
        }
        this.tracks[id].bytesCount = currentBytesCount;
        this.tracks[id].seconds = currentSeconds;
    }
    return bitrate;
};
//# sourceMappingURL=webrtc-stats.js.map